package tcp;

import java.net.*;
import java.io.*;

public class TCPServer {

	// The a listen socket to which Clients connect to
	public ServerSocket listenSocket = null;

	// The address to which the server will 'bind' to
	public InetSocketAddress address = null;

	/*
	 * Constructor will take in an address as input to bind to
	 */
	public TCPServer(InetSocketAddress address) {
		this.address = address;
	}

	/*
	 * Has all the logic of the server. It does the following: Creates a new
	 * ServerSocket object Binds the socket to the address we are given
	 * 
	 * Starts listening to requests indefinitely Accepts an incoming connection from
	 * a client (blocking method) Creates and starts a new thread that deals with
	 * this new client Rinse and repeat
	 * 
	 */
	public void startServer() {
		try {
			this.listenSocket = new ServerSocket();
			this.listenSocket.bind(address);

			System.out.println("TCP Server started!");

			while (true) {
				Socket serviceSocket = listenSocket.accept();
				Thread newServiceThread = new Thread(new Service(serviceSocket));
				newServiceThread.start();
			}
		}

		catch (Exception e) {
			e.printStackTrace();
		}

	}

	/*
	 * A class that can be run in a thread to service a client
	 */
	private class Service implements Runnable {
		Socket serviceSocket = null;
		ObjectInputStream in = null;
		ObjectOutputStream out = null;

		/*
		 * Sets the server, in and out attributes from the serviceSocket given
		 */
		private Service(Socket serviceSocket) {
			try {
				this.serviceSocket = serviceSocket;
				this.in = new ObjectInputStream(this.serviceSocket.getInputStream());
				this.out = new ObjectOutputStream(this.serviceSocket.getOutputStream());
				this.out.flush();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		/*
		 * The function that has the logic of the thread: Runs indefinitely expecting
		 * messages from the client We receive a message from the client from the "in"
		 * stream We write the message back to the client using th "out" stream
		 */
		public void run() {
			try {
				while (true) {
					/* Receive message from the client */
					String message = (String) in.readObject();
					System.out.println("Server Received:" + message);

					/* Echo the message back to the client */
					out.writeObject(message);
				}
			}

			catch (Exception e) {
				e.printStackTrace();
			}

			finally {
				if (serviceSocket != null)

					try {
						serviceSocket.close();
					}

					catch (Exception e) {
						e.printStackTrace();
					}
			}

		}
	}

	/*
	 * Create a new TCPServer with some port number Starts the server
	 */
	public static void main(String args[]) {
		int port = 3000;
		InetSocketAddress serverAddress = new InetSocketAddress(port);

		TCPServer server = new TCPServer(serverAddress);
		server.startServer();
	}
}
